from django import forms
from django.conf import settings


class LinkedFileInput(forms.FileInput):

    template_name = 'djeu/forms/widgets/linked-file-input.html'

    def format_value(self, value):
        """File input never renders a value."""
        if self.is_initial(value):
            return f'{settings.MEDIA_URL}{value}'

    def is_initial(self, value):
        """
        Return whether value is considered to be initial value.
        """
        return bool(value and getattr(value, 'url', False))

    def get_context(self, name, value, attrs):
        data_label = 'Загрузить'
        if 'button-label' in self.attrs:
            data_label = self.attrs['button-label']
        if 'data-label' in self.attrs:
            data_label = self.attrs['data-label']
        url = f'{settings.MEDIA_URL}{value}' if value else None
        return super(LinkedFileInput, self).get_context(name, value, {**attrs, **{
            'class': 'filepicker',
            'data-label': data_label,
            'url': url
        }})

    @property
    def media(self):
        extra = '' if settings.DEBUG else '.min'
        return forms.Media(
            js=(
                'admin/js/vendor/jquery/jquery%s.js' % extra,
                'djeu/js/jquery-filepicker.js',
                'djeu/js/jquery-filepicker.init.js',
            ) + (
                'admin/js/jquery.init.js',
            ),
            css={
                'screen': (
                    'djeu/css/filepicker.default.css',
                ),
            },
        )



from typing import Optional
from re import split as regexp_split
from django.forms.models import ModelMultipleChoiceField
from django.db import models


class ExtendedModelMultipleChoiceField(ModelMultipleChoiceField):

    def __init__(self, queryset, **kwargs):
        super().__init__(queryset, **kwargs)
        self.missed_keys_cache = dict()

    def label_from_instance(self, obj):
        """
        Convert objects into strings and generate the labels for the choices
        presented by this object. Subclasses can override this method to
        customize the display of the choices.
        """
        return str(obj)

    def to_python(self, value):
        return value

    def _default_procedure_to_missed_key(self, django_model, particular_value) -> Optional[int]:
        missed_key_key = f'{django_model._meta.model_name}.{particular_value}'.lower(
        )
        if missed_key_key in self.missed_keys_cache:
            return self.missed_keys_cache[missed_key_key]

        instance = django_model()
        tokens = regexp_split(r'\s+', particular_value)
        char_fields = [field for field in django_model._meta.fields if isinstance(
            field, models.CharField)]
        if len(tokens) <= len(char_fields):
            for i in range(0, len(tokens)):
                setattr(instance, char_fields[i].attname, tokens[i])
        elif len(char_fields) > 0:
            setattr(instance, char_fields[0].name, particular_value)
        else:
            raise BaseException(
                'Can\'t set automatically {particular_value} for {django_model}. Try to do in manual manner.')

        instance.save()
        self.missed_keys_cache[missed_key_key] = instance.pk
        return instance.pk

    def _compound_key_values(self, through, compound_key, additional_fields,
                             values, missed_key_procedure):
        result_count = 0
        result = None
        for value in values:  # value is comma searatd string
            compound_key_values = dict()
            previous_comma_position = 0
            comma_position = str(value).find(',')
            for particular_field_name in compound_key:
                particular_field_value = str(
                    value)[previous_comma_position:comma_position]
                pk_value = int(
                    particular_field_value) if particular_field_value.isdigit() else None
                if not pk_value and missed_key_procedure:
                    # For ForeignKey
                    django_model = through._meta.get_field(
                        particular_field_name).related_model
                    pk_value = missed_key_procedure(
                        django_model, particular_field_value)
                if not pk_value:
                    break
                compound_key_values[particular_field_name] = pk_value

                previous_comma_position = comma_position + 1
                comma_position = str(value).find(',', previous_comma_position)

            additional_fields_values = dict()
            for additional_field_name in additional_fields:
                particular_field_value = str(value)[previous_comma_position:] if comma_position == -1 else str(
                    value)[previous_comma_position:comma_position]
                additional_fields_values[additional_field_name] = particular_field_value

                previous_comma_position = comma_position + 1
                comma_position = str(value).find(',', previous_comma_position)

            # check full key
            if len(compound_key_values) == len(compound_key):
                stored_values = {**compound_key_values, **additional_fields_values}
                result_count += 1
                if result_count == 1:
                    result = stored_values
                elif result_count == 2:
                    for particular_field_name in [*compound_key_values, *additional_fields]:
                        result[particular_field_name] = [
                            result[particular_field_name], stored_values[particular_field_name]]
                else:
                    for particular_field_name in [*compound_key_values, *additional_fields]:
                        result[particular_field_name].append(
                            stored_values[particular_field_name])

        return result, result_count

    def clean(self, value):
        if self.widget and self.widget.field and self.widget.field.through:
            # Search by field. Eager approach
            through = self.widget.field.through
            compound_key = self.widget.field.key_data_components
            additional_fields = self.widget.field.additional_fields
            compound_values, compound_values_count = \
                self._compound_key_values(
                    through, compound_key, additional_fields, value, self._default_procedure_to_missed_key)
            # create relations
            if compound_values_count == 1:
                result = list(through.objects.filter(
                    **{k: v for k, v in compound_values.items()}))
                return result if result else [through(**{f'{k}_id' if k in compound_key else k: v for k, v in compound_values.items()})]
            elif compound_values_count > 1:
                result = list(through.objects.filter(
                    **{'%s__in' % k: v for k, v in compound_values.items()}))
                result_index = {
                    tuple([getattr(x, attname) for attname in compound_key]): x for x in result}
                # merge
                for i in range(0, compound_values_count):
                    compound_key_value = tuple(
                        [v[i] for k, v in compound_values.items()])
                    if not compound_key_value in result_index:
                        result.append(through(
                            **{**{f'{compound_key[i]}_id': compound_key_value[i] for i in range(0, len(compound_key))}}))
                return result
            else:
                return self.queryset.none()

        return super(ExtendedModelMultipleChoiceField, self).clean(value)



from django.dispatch import Signal

# contact_form_valid = Signal(
#     providing_args=['event', 'ip', 'site', 'sender_name', 'sender_email', 'email', 'subject', 'message']
# )

# contact_form_invalid = Signal(
#     providing_args=['event', 'ip', 'site', 'sender_name', 'sender_email']
# )



from __future__ import unicode_literals

import re

from django import forms

from django import VERSION
if VERSION >= (2, 0):
    from django.utils.translation import gettext_lazy as _
else:
    from django.utils.translation import ugettext_lazy as _

from ..conf import settings
# from ..models import Subject

from django.core.exceptions import ImproperlyConfigured

# try:
#    import bleach
# except ImportError:
#    raise ImproperlyConfigured('django-crispy-contact-form application requires bleach package')

from .julian_date_field import *


import os

from setuptools import setup, find_packages
from djeu import get_version as get_package_version

README = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
REQUIREMENTS = os.path.join(os.path.dirname(__file__), 'requirements.txt')
reqs = open(REQUIREMENTS).read().splitlines()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='djeu',
    version=get_package_version(),
    packages=find_packages(),
    include_package_data=True,
    install_requires=reqs,
    license='APACHE 2.0',
    description='Some widgets collections fro the Django 4.0+',
    long_description=README,
    url='https://github.com/usalko/djeu',
    author='usalko',
    author_email='ivict@rambler.ru',
    maintainer='usalko',
    maintainer_email='ivict@rambler.ru',
    zip_safe=False,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Development Status :: 4 - Beta',
    ],
)



#!/usr/bin/env python

import os
import sys

import django
from django.test.runner import DiscoverRunner as TestRunner

os.environ['DJANGO_SETTINGS_MODULE'] = 'test_settings'
test_dir = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
sys.path.insert(0, test_dir)


django.setup()


def runtests():
    test_runner = TestRunner(verbosity=1, interactive=True, failfast=False)
    failures = test_runner.run_tests(['contact_form'])
    if os.path.isfile('test.db'):
        os.unlink('test.db')
    return bool(failures)


if __name__ == '__main__':
    sys.exit(runtests())



from django import forms
from django.conf import settings


class DatePickerInput(forms.TextInput):

    template_name = 'djeu/forms/widgets/date-picker-input.html'

    def format_value(self, value):
        """File input never renders a value."""
        if self.is_initial(value):
            return f'{settings.MEDIA_URL}{value}'

    def is_initial(self, value):
        """
        Return whether value is considered to be initial value.
        """
        return bool(value and getattr(value, 'url', False))

    def get_context(self, name, value, attrs):
        context = super(DatePickerInput, self).get_context(name, value, {**attrs, **{
            'class': 'datepicker',
        }})
        #  Override string value
        context['widget']['value'] = value if isinstance(value, str) else context['widget']['value']
        return context

    @property
    def media(self):
        extra = '' if settings.DEBUG else '.min'
        return forms.Media(
            js=(
                'admin/js/vendor/jquery/jquery%s.js' % extra,
                'jquery-ui/jquery-ui%s.js' % extra,
                # 'djeu/js/jquery-datepicker.js',
                'djeu/js/i18n/datepicker-ru-RU.js',
                'djeu/js/jquery-datepicker.init.js',
            ) + (
                'admin/js/jquery.init.js',
            ),
            css={
                'screen': (
                    'jquery-ui/jquery-ui.min.css',
                ),
            },
        )
